// GpioSample.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <winioctl.h>
#include "pm.h"
#include "gpio.h"

void GPIOSetAsOutput(DWORD num, bool isOutputHigh)
{
	HANDLE hGPIO = GPIOOpen();
	DWORD mode = GPIO_DIR_OUTPUT;
	GPIOSetMode(hGPIO, num, mode);
	if(isOutputHigh)
		GPIOSetBit(hGPIO,num);
	else
		GPIOClrBit(hGPIO,num);
	GPIOClose(hGPIO);	
}

void GPIOSetAsInput(DWORD num)
{
	HANDLE hGPIO = GPIOOpen();
	DWORD mode = GPIO_DIR_INPUT;
	GPIOSetMode(hGPIO, num, mode);	
	GPIOClose(hGPIO);	
}

//return
BOOL IsGpioOutput(DWORD num)
{
	DWORD mode = 0;
	HANDLE hGPIO = GPIOOpen();	
	mode = GPIOGetMode(hGPIO,num);
	GPIOClose(hGPIO);
	return !(mode & GPIO_DIR_INPUT);
}

BOOL IsGpioInput(DWORD num)
{	
	return !IsGpioOutput(num);
}

BOOL IsGpioInputHigh(DWORD num)
{
	HANDLE hGPIO = GPIOOpen();	
	DWORD res = GPIOGetBit(hGPIO, num);
	GPIOClose(hGPIO);
	return res;
}

BOOL IsGpioInputLow(DWORD num)
{
	return !IsGpioInputHigh(num);
}


// Only works for OMAP GPIOs (not TWL GPIOs)
DWORD GpioSetupInterrupt(HANDLE hGPIO,DWORD num, HANDLE hEvent, bool interruptOnLow,bool interruptOnHigh,
						bool interruptOnFallingEdge, bool interruptOnRisingEdge)
{	
	DWORD mode = GPIOGetMode(hGPIO,num);
	// start by clearing all interrupts
	mode &= ~(GPIO_INT_LOW_HIGH | GPIO_INT_HIGH_LOW | GPIO_INT_LOW | GPIO_INT_HIGH);
	if(interruptOnHigh)
		mode |= GPIO_INT_HIGH;
	if(interruptOnLow)
		mode |= GPIO_INT_LOW;
	if(interruptOnFallingEdge)
		mode |= GPIO_INT_HIGH_LOW;
	if(interruptOnRisingEdge)
		mode |= GPIO_INT_LOW_HIGH;
	// enable interrupts
	mode |= GPIO_DIR_INPUT;
	GPIOSetMode(hGPIO,num,mode);
	
	// now setup bind the interrupt with the event
	return GPIOInterruptInitialize(hGPIO,num,hEvent);
}

#define INPUT_GPIO_NUM 170
#define OUTPUT_GPIO_NUM 140 
#define NUMBER_OF_INTERRUPT_EVENTS 1
#define SIMULATED_CLK_CYCLE_TIME		1000 //in mSecs
#define SIMULATED_CLK_NUMBER_OF_CYCLES	10 //mSecs

int _tmain(int argc, _TCHAR* argv[])
{
	DWORD counter = 0;
	HANDLE hGPIO = GPIOOpen();	

	
	/**
		This section demonstrates how to set s GPIO pin as output.
		We will set the voltage level of GPIO 140 (pin 14 on P9 in the
		SB_T35). If you measure the output with an oscilliscope, you will 
		see the 1Hz clock pattern (will last for 10 seconds)
	*/
	while(counter++ < SIMULATED_CLK_NUMBER_OF_CYCLES)
	{
		//Set 141 to low
		GPIOSetAsOutput(OUTPUT_GPIO_NUM,false);
		Sleep(SIMULATED_CLK_CYCLE_TIME/2);
		//Set 141 to high
		GPIOSetAsOutput(OUTPUT_GPIO_NUM,true);
		Sleep(SIMULATED_CLK_CYCLE_TIME/2);
	}

	/**
		This section demonstrates how to measure the input level on a GPIO.
		We use gpio 170, which is connected to pin 1 on P11 in the SB_T35.
	*/
	//First configure it as input
	GPIOSetAsInput(INPUT_GPIO_NUM);
	//Measure it's level and report it
	if(IsGpioInputHigh(INPUT_GPIO_NUM))
		printf("Gpio number %ld is high\r\n",INPUT_GPIO_NUM);
	else
		printf("Gpio number %ld is low\r\n",INPUT_GPIO_NUM);
	
	/** 
		Now we set up an interrupt on gpio 170.
		We configure it to report any edge (rising or falling).
		
		A similar code should appear in any IST (interrupt service thread) that waits for 
		a gpio event.

		The appication will stop when you create an edge event on GPIO 170
	*/
	HANDLE hGpioEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	GpioSetupInterrupt(hGPIO,INPUT_GPIO_NUM,hGpioEvent,false,false,true,true);
	counter = 0;
	while(counter++ < NUMBER_OF_INTERRUPT_EVENTS)
	{
		WaitForSingleObject(hGpioEvent, INFINITE);
		printf("Got an interrupt\r\n");
		GPIOInterruptDone(hGPIO,INPUT_GPIO_NUM);
	}
	GPIOClose(hGPIO);
	return 0;
}

